/* START LIBRARY DESCRIPTION *****************************
FILE:				DDS Comb Hardware.c
PARENT FILE:	DDS Comb.c
DESCRIPTION:	Hardware specific functions such as Rabbit
END DESCRIPTION *****************************************/

#ifndef DO_DEBUG
	#nodebug								// disables insertion of breakpoints for debugging, this file only.
#endif

/*** BeginHeader RabbitBoot */
void RabbitBoot (void);
void DdsBoot (void);
void DdsWrite (char *Werd);
void ImplementBuffers (void);
/*** EndHeader */


/*********************************** Boot the Rabbit **********************************************/
void RabbitBoot (void)  {			//Setup Rabbit RCM6760 I/O ports and registers.
/* START FUNCTION DESCRIPTION *****************************
RabbitBoot    <SSC Hardware.c>
DESCRIPTION:	Setup all Rabbit RCM6700 registers and ports
END DESCRIPTION ******************************************/
	/***  PORT A ***/																							  //not used.

	/***  PORT B ***/																			//PB[7,6,5,3] out to DDS.
	WrPortI (PBDDR, &PBDDRShadow, PBDDRShadow | 0xE8);									//PB[7,6,5,3] outputs.

	/***  PORT C ***/	  		 													 //PC[3] to display, [2:0] to DDS.
	WrPortI (PCDDR, &PCDDRShadow, PCDDRShadow | 0x0F);										 //PC[3:0] outputs.
	WrPortI (PCDCR, &PCDCRShadow, PCDCRShadow & 0xF0);							//PC[3:0] totem pole output.
	WrPortI (PCALR, &PCALRShadow, PCALRShadow & 0x1F);											 //PC[3] is TXC.
	BitWrPortI (PCFR, &PCFRShadow, 1, 3);										 //PC[3] is alternate function.

	/***  PORT D ***/																							  //not used.
	/***  PORT E ***/																							  //not used.
}


/*********************************** Boot the DDS *************************************************/
void DdsBoot (void)  {			//Setup the AD9958/9 dual/quad DDS.
/* START FUNCTION DESCRIPTION ************
DdsBoot    <DDS Comb Hardware.c>
DESCRIPTION:	Boot the DDS
END DESCRIPTION *************************/
	int Ch;
	char Werd[5];
																											//DDS HOUSEKEEPING.
	BitWrPortI (PBDR, &PBDRShadow, 1, 7);																//power down.
	BitWrPortI (PBDR, &PBDRShadow, 0, 3);															//IO_UPDATE low.
   BitWrPortI (PCDR, &PCDRShadow, 0, 1);                                    				 //clock low.
	BitWrPortI (PCDR, &PCDRShadow, 1, 2);																  //!CS high.
	BitWrPortI (PBDR, &PBDRShadow, 0, 7);																  //power up.
	BitWrPortI (PBDR, &PBDRShadow, 1, 6);													  //master reset high.
	BitWrPortI (PBDR, &PBDRShadow, 0, 6);													  //master reset low.
	BitWrPortI (PBDR, &PBDRShadow, 1, 4);										//SDIO3 high to reset SPI port.
	BitWrPortI (PBDR, &PBDRShadow, 0, 4);																 //SDIO3 low.

	Werd[0] = FR1;																					//FUNCTION REGISTER 1.
	Werd[1] = Bit2 + Bit4 + Bit7;	//(MS)							  //PLL divider = 5, sys clock > 255MHz.
	Werd[2] = Bit2 + Bit3;												 //enable ramp up/ramp down on SDIO_1-2.
	Werd[3] = Bit5;																	 //disable SYNC_CLK output pin.
	DdsWrite (Werd);

	Werd[0] = FR2;																					//FUNCTION REGISTER 2.
	Werd[1] = 0x00; //(MS)
	Werd[2] = 0x00;
	DdsWrite (Werd);

	Werd[0] = CSR;																			  //CHANNEL SELECT REGISTER.
	Werd[1] = 0xF0; //(MS)																		//select all channels.
	DdsWrite (Werd);

	Werd[0] = CFR;																			//CHANNEL FUNCTION REGISTER.
	Werd[1] = 0x00; //(MS)
	Werd[2] = Bit0 + Bit1;																				  //DAC scaling.
	Werd[3] = 0x00;
	DdsWrite (Werd);

/*	Werd[0] = CFTW0;																//CHANNEL FREQUENCY TUNING WORD 0.
	Werd[1] = 0x05; //(MS)																		  //FTW for 10MHz, MS.
	Werd[2] = 0x1E;
	Werd[3] = 0xB8;
	Werd[4] = 0x51;
	DdsWrite (Werd);*/

	Werd[0] = ACR;																		  //AMPLITUDE CONTROL REGISTER.
	Werd[1] = 0x00; //(MS)
	Werd[2] = 0x00;
	Werd[3] = 0x00;
	DdsWrite (Werd);

	ImplementBuffers();

	for (Ch = 0;  Ch < 4;  Ch++)  {							  //load the CFR shadow registers to defaults.
		CfrShadow[1][Ch] = 0x00;
		CfrShadow[2][Ch] = Bit0 + Bit1;
		CfrShadow[3][Ch] = 0x00;
	}
}


/**************************** Write a register in the DDS *****************************************/
void DdsWrite (char *Werd)  {			//Write to a register in the AD9958/9 dual/quad DDS.
/* START FUNCTION DESCRIPTION *********************************
DdsWrite    <DDS Comb Hardware.c>
DESCRIPTION:  Write a register in the AD9958/9 DDS
PARAMETER 1:  Address in Werd[0] then data in [1-3] as required
NOTES:        Using Single-bit Serial 2-Wire mode
END DESCRIPTION **********************************************/
	int Ctr, Bytes, Mask;

	switch (Werd[0])  {														 //GET TOTAL LENGTH OF WORD TO WRITE.
		case CSR:                           Bytes = 2;  break;			  		  //1 addr, 1 data byte.
		case FR2:  case CPOW0:  case LSRR:  Bytes = 3;  break;					  //1 addr, 2 data bytes.
		case FR1:  case CFR:    case ACR:   Bytes = 4;  break;					  //1 addr, 3 data bytes.
		default:                            Bytes = 5;  break;					  //1 addr, 4 data bytes.
	}
																										 //SETUP HOUSEKEEPING.
	BitWrPortI (PCDR, &PCDRShadow, 0, 2);															  //set !CS low.
	BitWrPortI (PBDR, &PBDRShadow, 1, 5);			  //force SDIO_3 low to not reset SPI state machine.

	for (Ctr = 0;  Ctr < Bytes; Ctr++)  {													  //BIT BANG THE BITS.
		Mask = 0x80;
	   do  {
	      if (Werd[Ctr] & Mask)  BitWrPortI (PCDR, &PCDRShadow, 1, 0);			  //set data bit high.
	      else                   BitWrPortI (PCDR, &PCDRShadow, 0, 0);			  //set data bit low.
	      BitWrPortI (PCDR, &PCDRShadow, 1, 1);                                     //set clock high.
	      BitWrPortI (PCDR, &PCDRShadow, 0, 1);                                     //set clock low.
	      Mask >>= 1;                                                    //shift mask to next LS bit.
	   } while (Mask);
	}
	 																									  //EXIT HOUSEKEEPING.
	BitWrPortI (PBDR, &PBDRShadow, 0, 5);											  //allow BNC to set SDIO_3.
	BitWrPortI (PCDR, &PCDRShadow, 1, 2);															 //set !CS high.
}



/************************ Implement the serial buffers to live data *******************************/
void ImplementBuffers (void)  {
	BitWrPortI (PBDR, &PBDRShadow, 1, 3);														  //IO_UPDATE high.
	BitWrPortI (PBDR, &PBDRShadow, 0, 3);														  //IO_UPDATE low.
}
/**************************************************************************************************/


/*********************************** Help descriptions from SSC.c *********************************/
//Dynamic C can't read a help description in the root file, for some dumb reason.
/* START FUNCTION DESCRIPTION *******************************************
Pause    <DDS Comb.c>
DESCRIPTION: Stop and wait "mS" interval before leaving this function.
PARAMETER:	 int mS; time in mS, 1 to 32,767
NOTE:        Timing 1mS because can start anywhere in the RTC mS period.
END DESCRIPTION ********************************************************/
/**************************************************************************************************/

