/* START LIBRARY DESCRIPTION ****************************************************
FILE: 			Display NHD-0220D3Z.c
DESCRIPTION:	Functions for Newhaven NHD-0220D3Z-xxx-xxx 2-line 20 char display
END DESCRIPTION ****************************************************************/


/********* Function PROTOTYPES (follow syntax, including comment lines) ***************************/
/*** BeginHeader DisplayBoot, DisplayWrite */
void DisplayBoot(void);
void DisplayWrite (char *Line1, char *Line2, int Brightness);
void DisplayContrast (char* Contrast);

#define BRIGHT 	8	  									 //when connected to host, display brightness (1-8).
#define DIM    	2									//when not connected to host, display brightness (1-8).
#define CONTRAST 	42							  							 //Set default contrast (1-50, 50=high).
/*** EndHeader */

#ifndef DO_DEBUG
	#nodebug							   // disables insertion of breakpoints for debugging, this file only.
#endif


/*********************************** Function DisplayBoot *****************************************/
void DisplayBoot (void)  {	//boot the Newhaven Displays NHD-0220D3Z-xxx-xxx (2-line, 20 char)
	serCopen (9600);																	//CONFIGURE RABBIT SERIAL PORT.
	serCdatabits (PARAM_8BIT);
	serCparity (PARAM_NOPARITY);

	serCputc(0xFE);  serCputc(0x52);  serCputc(CONTRAST);	 									 //SET CONTRAST.

				 /*NB: add following precompiler directives to assign Tx port and increased buffer size.
	#define CDRIVE_TXD    3		//serial port C must use PC3 only.
	#define COUTBUFSIZE 127		//serial port C Tx buffer.
	#define CINBUFSIZE  127		//serial port C Rx buffer.  */
}


/*********************************** Write to the Display *****************************************/
void DisplayWrite (char *Line1, char *Line2, int Brightness)  {
/* START FUNCTION DESCRIPTION *********************************************
DisplayWrite       <Display NHD-0220D3Z.c>
DESCRIPTION: With the Newhaven Displays NHD-0220D3Z-xxx-xxx, display 2
	lines of 20 characters.  Note that there must be a pause of at least a
	few mS before setting brightness.
ENTER:  Pointers to line 1 and line 2 null terminated strings.
        Also, backlighting from 1 to 8.  No error checking is done to
        string length or brightness level!
END DESCRIPTION **********************************************************/
	serCputc(0xFE);  serCputc(0x53);  serCputc(Brightness);							  //ADJUST BRIGHTNESS.
	Pause(10);

	serCputc(0xFE);  serCputc(0x51);																	//clear display.
	Pause(10);
	serCwrite(Line1, strlen(Line1));															 //display first line.
	Pause(10);
	serCputc(0xFE);  serCputc(0x45);  serCputc(0x40);						  //go to start of second line.
	Pause(10);
	serCwrite(Line2, strlen(Line2));															//display second line.
	Pause(10);
}


/*********************************** Adjust Display Contrast **************************************/
void DisplayContrast (char* Contrast)  {
/* START FUNCTION DESCRIPTION ********************
DisplayContrast       <Display NHD-0220D3Z.c>
DESCRIPTION:	Adjust the display contrast
PARAMETER 1:	Contrast, 0 to 99.  This display
					can only use 1 to 50
END DESCRIPTION *********************************/
	unsigned int Con;

	Con = (unsigned int) Contrast[0];
	Con /= 2;										 //adjust user's 0 to 99 range to display's 1 to 50 range.
	if (Con > 50)  Con = 50;
	if (!Con)      Con = 1;
	serCputc(0xFE);  serCputc(0x52);  serCputc(Con);		 									 //SET CONTRAST.
	Pause(10);
}
/**************************************************************************************************/

